{ config, pkgs, lib, ... }:

{
  imports = [
    ./hardware-configuration.nix
    ../server.nix
  ];

  nixpkgs.hostPlatform = "x86_64-linux";

  networking = {
    hostId = "08bf6db3";
    firewall = {
      enable = true;
      allowedTCPPorts = [
        22    # SSH
      ];
    };
  };

  users = {
    users = {
      jellyfin = {
        isSystemUser = true;
        group = "jellyfin";
        shell = "/bin/sh";
        openssh.authorizedKeys.keys = [
          "ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAIOaaS+KMAEAymZhIJGC4LK8aMhUzhpmloUgvP2cxeBH4 jellyfin"
        ];
      };
      root = {
        openssh.authorizedKeys.keys = [
          "ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAILVHTjsyMIV4THNw6yz0OxAxGnC+41gX72UrPqTzR+OS jordan@vimium.com"
        ];
      };
    };
    groups = {
      jellyfin = { };
    };
    extraGroups.acme.members = [ "kanidm" "nginx" ];
  };

  services.openssh.settings.PermitRootLogin = lib.mkForce "prohibit-password";

  security.acme.certs."auth.vimium.com" = {
    postRun = "systemctl restart kanidm.service";
    group = "acme";
  };

  services.kanidm = let
    baseDomain = "vimium.com";
    domain = "auth.${baseDomain}";
    uri = "https://${domain}";
  in {
    package = pkgs.unstable.kanidm;
    enableClient = true;
    enableServer = true;
    clientSettings = {
      inherit uri;
    };
    serverSettings = {
      bindaddress = "[::1]:3013";
      ldapbindaddress = "[::1]:636";
      domain = baseDomain;
      origin = uri;
      tls_chain = "${config.security.acme.certs.${domain}.directory}/full.pem";
      tls_key = "${config.security.acme.certs.${domain}.directory}/key.pem";
    };
  };

  services.nginx.virtualHosts = {
    "auth.vimium.com" = {
      useACMEHost = "auth.vimium.com";
      forceSSL = true;
      locations."/" = {
        proxyPass = "https://[::1]:3013";
      };
    };
  };

  modules = rec {
    databases.postgresql.enable = true;
    services = {
      borgmatic = {
        enable = true;
        directories = [
          "/home"
          "/var/lib"
          "/var/www"
        ];
        repoPath = "ssh://p91y8oh7@p91y8oh7.repo.borgbase.com/./repo";
      };
      coturn = {
        enable = true;
        realm = "turn.vimium.com";
        matrixIntegration = true;
      };
      gitea.enable = true;
      headscale.enable = true;
      matrix = {
        enable = true;
        bridges = {
          signal = true;
          whatsapp = true;
        };
        usePostgresql = databases.postgresql.enable;
      };
      nginx.enable = true;
      photoprism.enable = true;
    };
  };

  system.stateVersion = "22.11";
}
